//SWITCH-TO: vue
import { createVNode, defineComponent, type PropType } from "vue"
import jsmind from "jsmind"
import 'jsmind/style/jsmind.css'
import tools from "../tools"
import type { subject_item_t, jsmind_node_t } from "../tools"
import { Modal } from "ant-design-vue"



//jsmind文档 https://hizzgdev.github.io/jsmind/docs/zh/3.operation.html
export default defineComponent({
    components: {
    },
    props: {
        subject: {
            type: Object as PropType<subject_item_t>,
            required: true,
        },

        max_level: {
            type: Number,
            default: 3,
        },
    },
    emits: [ "update_subject"],
    setup() {
        return {
        }
    },
    data() {
        return {
            jm: <any>undefined,
            cur_edit_node_id: "",
            next_id: 0,
            node_tree_data: <jsmind_node_t>{},
            drag_node_id: "",
            edit_content_dlg: {
                open: false,
                topic: "",
                content: "",
            },
            download_id: 0,
            ppt_title: <any>'',
            gen_list: <any>[],
            percent: 0,
            spinning: false,
        };
    },
    watch: {
        subject(_val: boolean) {
            document.getElementById("jsmind_container")!.innerHTML = "";
            this.node_tree_data = tools.subject_to_jsmind_node(this.subject);
            this.init_data()
        }

    },

    created() {
        this.node_tree_data = tools.subject_to_jsmind_node (this.subject);
    },
    mounted() {
        this.init_data();
    },
    methods: {
        init_data() {
            var mind = {
                /* 元数据，定义思维导图的名称、作者、版本等信息 */
                "meta": {
                    "name": this.subject.title + "思维导图",
                    "author": "mindshow",
                    "version": "0.2"
                },
                /* 数据格式声明 */
                "format": "node_tree",
                /* 数据内容 */
                "data": this.node_tree_data,
                "name": "text",

            };
            var options = {
                container: 'jsmind_container',
                theme: "greensea",
                editable: false,


                view: {
                    engine: 'canvas',   // 思维导图各节点之间线条的绘制引擎
                    hmargin: 100,        // 思维导图距容器外框的最小水平距离
                    vmargin: 50,         // 思维导图距容器外框的最小垂直距离
                    line_width: 1,       // 思维导图线条的粗细
                    line_color: '#53ee62',  // 思维导图线条的颜色
                    // line_color: '#555',  // 思维导图线条的颜色
                    line_style: 'curved',// 思维导图线条的样式，直线(straight)或者曲线(curved)
                    draggable: false,   // 当容器不能完全容纳思维导图时，是否允许拖动画布代替鼠标滚动
                    hide_scrollbars_when_draggable: false, // 当设置 draggable = true 时，是否隐藏滚动条
                    node_overflow: 'hidden', // 节点文本过长时的样式
                    zoom: {             // 配置缩放
                        min: 0.5,       // 最小的缩放比例
                        max: 2.1,       // 最大的缩放比例
                        step: 0.1,      // 缩放比例间隔
                    },
                    custom_node_render: this.custom_node_render, // 自定义的节点渲染方法
                },

                shortcut: {
                    enable: false,        // 是否启用快捷键
                    handles: {},         // 命名的快捷键事件处理器
                    mapping: {           // 快捷键映射
                        addchild: [45, 4096 + 13],  // <Insert>, <Ctrl> + <Enter>
                        addbrother: 13,    // <Enter>
                        editnode: 113,   // <F2>
                        delnode: 46,    // <Delete>
                        toggle: 32,    // <Space>
                        left: 37,    // <Left>
                        up: 38,    // <Up>
                        right: 39,    // <Right>
                        down: 40,    // <Down>
                    }
                },
            };
            this.jm = new jsmind(options);

            this.jm.show(mind);
        },


        //定制节点
        custom_node_render(_jm: any, element: HTMLElement, node: jsmind_node_t) {
            // console.log("node",node );
            element.style.minWidth = "120px";
            element.style.width = "250px";
            element.style.overflow = "visible";
            element.innerHTML = "";

            element.addEventListener("change", (_e: Event) => {
                if (_e.target) {
                    // (<any>_e.target).addEventListener('blur', (_e: any) => {
                    //     this.save_local();
                    // })
                }
            });

            if (!node.isroot) {
                element.draggable = true;
                element.addEventListener("dragover", (e) => {
                    if (this.drag_node_id != node.id) {
                        var from_node = this.jm.get_node(this.drag_node_id);
                        if (from_node?.parent == node?.parent) {
                            e.preventDefault();
                        }
                    }
                });
                element.addEventListener("drop", (_e) => {
                    var from_node: jsmind_node_t = this.jm.get_node(this.drag_node_id);
                    if (from_node) {
                        this.jm.enable_edit();
                        console.log("move ", from_node.parent?.children[0]?.id, from_node.id);

                        var from_node_index = from_node.parent?.children.findIndex((item) => {
                            return item.id == from_node.id;
                        });
                        var node_index = node.parent?.children.findIndex((item) => {
                            return item.id == node.id;
                        });
                        console.log("from_node_index", from_node_index, node_index);


                        if (node_index !== undefined && from_node_index == node_index - 1) { //
                            this.jm.move_node(node, from_node.id);
                        } else {
                            this.jm.move_node(from_node, node.id);
                        }
                        this.jm.disable_edit();
                        this.drag_node_id = "";
                    }
                });


                element.addEventListener("dragstart", (_e) => {
                    //不知为何dataTransfer无效
                    //e.dataTransfer!.setData("text", node.id);
                    //console.log("START: set"  ,node.id, e );
                    this.drag_node_id = node.id;

                    // e.dataTransfer!.setData('text', node.id);
                });

            }
            // element.appendChild()
            var item = document.createElement("div");
            item.style.position = "relative";
            item.addEventListener("click", () => {
                if (node.id != this.cur_edit_node_id) {
                    this.cur_edit_node_id = "";
                    this.jm.disable_edit();
                }
            });
            var text_item = document.createElement("div");
            // text_item.style.overflow = "hidden";
            // text_item.style.textOverflow = "ellipsis";
            text_item.style.whiteSpace = "pre-wrap";
            if (this.check_end_node(node)) {
                text_item.innerHTML = `<div class="c-title">${ node.topic }</div> <div class="c-content">` + tools.html_escape(node.data.content) + '</div>';
            } else {
                text_item.innerText = node.topic;
            }

            var opt_item = document.createElement("div");

            opt_item.className = "opt-list";
            if (element.draggable) {

                //编辑
                var opt_drag = document.createElement("div");
                opt_drag.className = "opt";
                opt_drag.title = "拖动";
                opt_drag.innerHTML = `<svg t="1704256494042" viewBox="0 0 1024 1024" version="1.1" xmlns="http://www.w3.org/2000/svg" p-id="9524" data-spm-anchor-id="a313x.search_index.0.i1.68f83a81kgrNR1"   width="1em" height="1em" fill="currentColor"><path d="M469.333333 256a85.333333 85.333333 0 1 1-85.333333-85.333333 85.333333 85.333333 0 0 1 85.333333 85.333333z m-85.333333 170.666667a85.333333 85.333333 0 1 0 85.333333 85.333333 85.333333 85.333333 0 0 0-85.333333-85.333333z m0 256a85.333333 85.333333 0 1 0 85.333333 85.333333 85.333333 85.333333 0 0 0-85.333333-85.333333z m256-341.333334a85.333333 85.333333 0 1 0-85.333333-85.333333 85.333333 85.333333 0 0 0 85.333333 85.333333z m0 85.333334a85.333333 85.333333 0 1 0 85.333333 85.333333 85.333333 85.333333 0 0 0-85.333333-85.333333z m0 256a85.333333 85.333333 0 1 0 85.333333 85.333333 85.333333 85.333333 0 0 0-85.333333-85.333333z" p-id="9525"></path></svg>`;

                opt_item.appendChild(opt_drag);

            }

            //编辑
            var opt_edit = document.createElement("div");
            opt_edit.className = "opt";
            opt_edit.title = "编辑";
            opt_edit.innerHTML = `<svg t="1703644978408" class="icon" viewBox="0 0 1024 1024" version="1.1" xmlns="http://www.w3.org/2000/svg" p-id="9158"  width="1em" height="1em" fill="currentColor"><path d="M904.448 316.8L696.96 109.248 742.144 64l207.552 207.552-45.248 45.248zM382.208 839.04l-231.232 46.272a19.2 19.2 0 0 1-22.592-22.592l46.272-231.232 467.008-467.008 207.616 207.552-467.072 467.072z m376.512-467.008l-117.056-117.056L233.6 663.04l-29.312 146.368 146.368-29.248 408.064-408.128zM960 704h-256v64h256v-64zM960 832H544v64H960v-64z" p-id="9159"></path></svg>`;
            opt_item.appendChild(opt_edit);

            opt_edit.addEventListener("click", () => {
                this.node_edit(node)
            });


            if (!this.check_end_node(node)) {
                //新增子节点
                var opt_add_child = document.createElement("div");
                opt_add_child.className = "opt";
                opt_add_child.title = "增加子节点";
                opt_add_child.innerHTML = `<svg t="1703643822092" style="color:#FFF;" viewBox="0 0 1024 1024" version="1.1" xmlns="http://www.w3.org/2000/svg" p-id="9005" width="1em" height="1em" fill="currentColor" ><path d="M470.4 553.6v246.4h83.2V553.6h246.4v-83.2H553.6V224h-83.2v246.4H224v83.2h246.4z" p-id="9006"></path></svg>`;
                opt_add_child.addEventListener("click", () => {
                    this.node_add(node)
                });
                opt_item.appendChild(opt_add_child);
            }

            if (!node.isroot) {
                //删除
                var opt_del = document.createElement("div");
                opt_del.className = "opt";
                opt_del.title = "删除";
                opt_del.innerHTML = `<svg t="1703647348014" class="icon" viewBox="0 0 1024 1024" version="1.1" xmlns="http://www.w3.org/2000/svg" width="1em" height="1em" fill="currentColor" p-id="9311" ><path d="M384 768V384h64v384H384zM576 384v384h64V384H576z" p-id="9312"></path><path d="M672 192H896v64h-64v640a64 64 0 0 1-64 64H256a64 64 0 0 1-64-64V256H128V192h224V115.2a51.2 51.2 0 0 1 51.2-51.2h217.6a51.2 51.2 0 0 1 51.2 51.2V192z m-256 0h192V128h-192v64zM256 256v640h512V256H256z" p-id="9313"></path></svg>`;
                opt_item.appendChild(opt_del);

                opt_del.addEventListener("click", () => {
                    this.node_del(node)
                });

            }


            (<any>opt_item)["data-id"] = node.id;

            item.appendChild(text_item);
            item.appendChild(opt_item);
            element.appendChild(item);
            return true;


        },
        //是否是子节点
        check_end_node(node: jsmind_node_t) {
            return node.data?.level >= this.max_level;
        },
        node_edit(node: jsmind_node_t) {
            this.cur_edit_node_id = node.id;
            if (this.check_end_node(node)) {
                this.edit_content_dlg.open = true;
                this.edit_content_dlg.topic = node.topic;
                this.edit_content_dlg.content = node.data.content;
                //
            } else {
                this.jm.enable_edit();
                this.jm.begin_edit(node);
            }
        },
        node_del(node: jsmind_node_t) {
            Modal.confirm({
                title: '删除节点',
                content: createVNode('div', {style: 'font-size:16px;'},
                    ['要删除[',
                        createVNode('div', {style: 'font-weight:bold; display:inline-block;'},
                            node.topic), "]?"]
                ),
                onOk: () => {

                    this.jm.enable_edit();
                    this.jm.remove_node(node.id);
                    this.jm.disable_edit();
                },
                onCancel() {
                },
            });

        },


        node_add(node: jsmind_node_t) {
            this.jm.enable_edit();
            this.jm.add_node(node, tools.get_next_id(), "新节点", {
                level: node.data.level + 1,
                content: "",
            }, "right");
            console.log("ddddd", "新增")
            this.jm.disable_edit();

        },
        set_cur_content() {
            var node: jsmind_node_t = this.jm.get_node(this.cur_edit_node_id);
            if (node) {
                this.jm.enable_edit();
                node.data.content = this.edit_content_dlg.content;
                this.jm.update_node(this.cur_edit_node_id, this.edit_content_dlg.topic);
                this.jm.disable_edit();

                this.edit_content_dlg.open = false;

            }

        },
        get_subject(){
            return  tools.jsmind_node_to_subject (this.jm.get_root());
        },

        update_tree_data(is_update = true) {
            let subject = this.subject
            if (is_update) {
                subject= this.get_subject();
            }
            this.$emit("update_subject", subject, true)
        }
    }
})
;
