import m2j from "./md2json/md2json";


export interface jsmind_node_t {
    id: string
    topic: string
    isroot?: boolean,
    parent?: jsmind_node_t,
    index: number,
    children: jsmind_node_t[],
    data: {
        level: number, //
        content: string,
        image_url: any[]
    },
};


export type subject_item_t = {
    title?: string, //节点标题
    list?: subject_item_t[], //下级列表
    content?: string, //节点内容
};

var tools ={

    //markdown 转成结构化数据
    md_to_subject(md_content: string) {
        var md_tree_data: any = {};
        try {
            md_tree_data = m2j.parse(md_content);
        } catch {
            return undefined;
        }

        var gen_subject_list = (tree_node: any) => {

            var sub_list: subject_item_t[] = [];
            var content = <any>undefined;
            for (var sub_title in tree_node) {
                if (sub_title == "raw") {
                    content = tree_node["raw"];
                    continue;
                }
                var sub_item: subject_item_t = {
                    title: sub_title,
                };
                if ( typeof (tree_node[sub_title]) === "object") {
                    var sub_ret = gen_subject_list(tree_node[sub_title]);
                    sub_item.content = sub_ret.content;
                    sub_item.list = sub_ret.list;
                }
                sub_list.push(sub_item);
            }
            return {
                list: sub_list,
                content: content,
            };
        };
        var ret = gen_subject_list(md_tree_data);
        return ret.list[0];
    },

    //结构化数据 转成 markdown
    subject_to_md(subject: subject_item_t) {
        var gen_text = (item: subject_item_t, level_str = "#") => {
            var lines: string[] = [];
            lines.push(level_str + ' ' + item.title);
            if (item.content) {
                lines.push(item.content);
            }

            var sub_fix = level_str + "#";
            item.list?.forEach((sub_item,) => {
                var sub_lines = gen_text(sub_item, sub_fix);
                lines.push(...sub_lines);
            });
            return lines;
        };
        return gen_text(subject).join("\n");
    },
    next_id:0,
    get_next_id() {
        this.next_id++;
        return `id_${this.next_id}`;
    },

    // 将大纲结构转成 jsmind 要求的格式
    subject_to_jsmind_node(tree_data: subject_item_t, level = 0, max_level=3): jsmind_node_t {
        var node: any = {
            id: this.get_next_id(),
            topic: tree_data.title,
            level: level,
        };

        if (level < max_level) {

            if (level === 2 && tree_data.list && tree_data.list.length === 0) {
                tree_data.list.push({
                    title: tree_data.title,
                    content: tree_data.content,
                })
            }

            node.children = tree_data.list!.map((sub) => {
                return this.subject_to_jsmind_node(sub, level + 1 )
            });
        } else {
            node.content = tree_data.content;
        }
        return node;
    },

    // 将 jsmind 转成  大纲结构
    jsmind_node_to_subject(node: jsmind_node_t, max_level = 3) {

        var gen_data: (node: jsmind_node_t) => subject_item_t;
        gen_data = (node: jsmind_node_t) => {
            if (node.data?.level >= max_level) {
                return <subject_item_t>{
                    content: node.data.content,
                    title: node.topic,
                    list: [],
                    image_url: node.data.image_url
                }
            } else {
                return <subject_item_t>{
                    title: node.topic,
                    list: node.children.map((sub_node) => {
                        return gen_data(sub_node);
                    }),
                    image_url: node.data.image_url
                }

            }
        }
        return gen_data( node);
    },

    html_escape(str: string) {
        var temp = document.createElement("div");
        temp.innerText = str;
        return temp.innerHTML;
    },



};
export default tools;
